#include "stdafx.h"

#include "KalmanFilter.h"
#include <iostream>


KalmanFilter::KalmanFilter()
:m_alreadyBegun(false)
{
	m_timeLastMeasurement = clock();
	int dynam_params = 4; // x,y,dx,dy
	int measure_params = 2;
	m_kalman = cvCreateKalman(dynam_params, measure_params);
	m_state = cvCreateMat( dynam_params, 1, CV_32FC1 );

	// random number generator stuff
	cvRandInit( &m_rng, 0, 1, -1, CV_RAND_UNI );
	cvRandSetRange( &m_rng, 0, 1, 0 );
	m_rng.disttype = CV_RAND_NORMAL;
	cvRand( &m_rng, m_state );

	m_processNoise = cvCreateMat( dynam_params, 1, CV_32FC1 ); // (w_k)
	m_measurement = cvCreateMat( measure_params, 1, CV_32FC1 ); // two parameters for x,y  (z_k)
	//m_measurementNoise = cvCreateMat( measure_params, 1, CV_32FC1 ); // two parameters for x,y (v_k)
	cvZero(m_measurement);

	// F matrix data
	// F is transition matrix.  It relates how the states interact
	// For single input fixed velocity the new value
	// depends on the previous value and velocity- hence 1 0 1 0
	// on top line. New velocity is independent of previous
	// value, and only depends on previous velocity- hence 0 1 0 1 on second row
	float dt = 0.01;
	const float F[] = {
		1, 0, dt, 0, //x + dt
		0, 1, 0, dt, //y + dt
		0, 0, 1, 0, //dx = dx
		0, 0, 0, 1, //dy = dy
	};
	memcpy( m_kalman->transition_matrix->data.fl, F, sizeof(F));
	cvSetIdentity( m_kalman->measurement_matrix, cvRealScalar(1) ); // (H)
	cvSetIdentity( m_kalman->process_noise_cov, cvRealScalar(1e-5) ); // (Q)
	cvSetIdentity( m_kalman->measurement_noise_cov, cvRealScalar(1e-1) ); // (R)
	cvSetIdentity( m_kalman->error_cov_post, cvRealScalar(1));

	// choose random initial state
	cvRand( &m_rng, m_kalman->state_post );
}


KalmanFilter::~KalmanFilter()
{
	// Destroy state, process_noise, and measurement matricies
	cvReleaseMat( &m_state );
	cvReleaseMat( &m_processNoise );
	cvReleaseMat( &m_measurement );
	//cvReleaseMat( &m_measurementNoise );
	cvReleaseKalman( &m_kalman );
}


void KalmanFilter::predictionBegin(float x, float y)
{
    // Initialize kalman variables (change in x, change in y,...)
    // m_kalman->state_post is prior state
    m_kalman->state_post->data.fl[0] = x; // x
    m_kalman->state_post->data.fl[1] = y; // y
    m_kalman->state_post->data.fl[2] = 0.0; //dx
    m_kalman->state_post->data.fl[3] = 0.0; //dy
	m_alreadyBegun = true;
}


void KalmanFilter::predictionUpdate(float x, float y)
{
	m_timeLastMeasurement = clock();
	
	m_state->data.fl[0] = x; // x
	m_state->data.fl[1] = y; // y
	m_haveMeasurement = true;
}


void KalmanFilter::predictionReport()
{
	clock_t timeCurrent = clock();

	const CvMat* prediction = cvKalmanPredict( m_kalman, 0 );

	// If we have received the real position recently, then use that position to correct
	// the kalman filter.  If we haven't received that position, then correct the kalman
	// filter with the predicted position
	if (m_haveMeasurement)
		m_haveMeasurement = false;

	// The filter may run much faster than the measurements are coming in.
	// So this allows the filter to pick up from the last predicted state and continue in between measurement
	else 
	// update with the predicted position
	{
		m_state->data.fl[0] = prediction->data.fl[0];
		m_state->data.fl[1] = prediction->data.fl[1];
	}

	// generate measurement noise(z_k)
	cvRandSetRange( &m_rng, 0, sqrt(m_kalman->measurement_noise_cov->data.fl[0]), 0 );
	cvRand( &m_rng, m_measurement );

	// zk = Hk * xk + vk
	// measurement = measurement_error_matrix * current_state + measurement_noise
	cvMatMulAdd( m_kalman->measurement_matrix, m_state, m_measurement, m_measurement );

	// adjust Kalman filter state
	cvKalmanCorrect( m_kalman, m_measurement );

	// generate process noise(w_k)
	cvRandSetRange( &m_rng, 0, sqrt(m_kalman->process_noise_cov->data.fl[0]), 0 );
	cvRand( &m_rng, m_processNoise );

	// xk = F * xk-1 + B * uk + wk
	// state = transition_matrix * previous_state + control_matrix * control_input + noise
	cvMatMulAdd( m_kalman->transition_matrix, m_state, m_processNoise, m_state );
}